import type { HighlighterCore, LanguageRegistration, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { createContext, useCallback, useContext, useEffect, useMemo, useState } from 'react'

interface ShikiContextValue {
  highlighter: HighlighterCore | null
  isReady: boolean
  error: Error | null
  registerLanguage: (lang: LanguageRegistration) => Promise<void>
  registerTheme: (theme: ThemeRegistrationAny) => Promise<void>
  getLoadedLanguages: () => string[]
  getLoadedThemes: () => string[]
}

interface ShikiProviderProps {
  children: React.ReactNode
  initialLanguages?: LanguageRegistration[]
  initialThemes?: ThemeRegistrationAny[]
}

const ShikiContext = createContext<ShikiContextValue | null>(null)

// Default empty arrays as stable references
const DEFAULT_LANGUAGES: LanguageRegistration[] = []
const DEFAULT_THEMES: ThemeRegistrationAny[] = []

export function ShikiProvider({ children, initialLanguages = DEFAULT_LANGUAGES, initialThemes = DEFAULT_THEMES }: ShikiProviderProps) {
  const [highlighter, setHighlighter] = useState<HighlighterCore | null>(null)
  const [isReady, setIsReady] = useState(false)
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    const initializeHighlighter = async () => {
      try {
        const instance = await createHighlighterCore({
          engine: createJavaScriptRegexEngine(),
          langs: initialLanguages,
          themes: initialThemes,
        })
        setHighlighter(instance)
        setIsReady(true)
      }
      catch (err) {
        setError(err instanceof Error ? err : new Error('Failed to initialize highlighter'))
      }
    }

    initializeHighlighter()
  }, [initialLanguages, initialThemes])

  const registerLanguage = useCallback(
    async (lang: LanguageRegistration) => {
      if (!highlighter)
        return
      await highlighter.loadLanguage(lang)
    },
    [highlighter],
  )

  const registerTheme = useCallback(
    async (theme: ThemeRegistrationAny) => {
      if (!highlighter)
        return
      await highlighter.loadTheme(theme)
    },
    [highlighter],
  )

  const getLoadedLanguages = useCallback(() => {
    return highlighter ? highlighter.getLoadedLanguages() : []
  }, [highlighter])

  const getLoadedThemes = useCallback(() => {
    return highlighter ? highlighter.getLoadedThemes() : []
  }, [highlighter])

  const value = useMemo(
    () => ({
      highlighter,
      isReady,
      error,
      registerLanguage,
      registerTheme,
      getLoadedLanguages,
      getLoadedThemes,
    }),
    [highlighter, isReady, error, registerLanguage, registerTheme, getLoadedLanguages, getLoadedThemes],
  )

  return <ShikiContext.Provider value={value}>{children}</ShikiContext.Provider>
}

export function useShiki() {
  const context = useContext(ShikiContext)
  if (!context) {
    throw new Error('useShiki must be used within a ShikiProvider')
  }
  return context
}

export type { ShikiContextValue, ShikiProviderProps }
